!     Last change:  WRG  28 Jul 2006    4:05 pm
PROGRAM mainprogram
!================================================================================!
! **** LAST MODIFIED: 3 May 2005 ****                                            !
! **** WAYNE-ROY GAYLE, JEAN-FRANCOIS RICHARD, UNIVERSITY OF PITTSBURGH ****     !
!--------------------------------------------------------------------------------!
! This program computes independent private values asymmetric first-price        !
! auctions where the auctioneer sets a reserve price. Asymmetry arises from      !
! the specification of ex-ante heterogenous distributions of private values,     !
! as well as from collusion among subsets of homogenous bidders.                 !
! This program also computes the optimal reserve price under the first price     !
! and the second price auction schemes.                                          !
! Furthermore, the program provides as output the expected revenue to the        !
! auctioneer and the expected suprlus to each bidder for both the first price    !
! and second price auction schemes.                                              !
!================================================================================!
 USE mpreamble
 USE cdf_mod
 IMPLICIT NONE
 INTEGER(4)                     :: i,j,optres,bestr,redo
 REAL(8)                        :: objs
 REAL(8),ALLOCATABLE,DIMENSION(:,:)       :: ta
 INTEGER                        :: hertz, clock0, clock1
 REAL                           :: time
 INTRINSIC SYSTEM_CLOCK

 WRITE(*,80)
 80 FORMAT (' ',/, &
 1X,'This program computes independent private values asymmetric first price',/, &
 1X,'auctions and procurements, where the auctioneer sets a reserve price.',/, &
 1X,'Asymmetry arises from the specification of ex-ante heterogenous distributions',/, &
 1X,'of private values as well as from collusion among subsets of homogenous and/or',/, &
 1X,'heterogenous bidders. This program also computes the optimal reserve price',/, &
 1X,'under the first price and the second price schemes.',/, &
 1X,'Furthermore, the program provides as output the expected revenue to the',/, &
 1X,'auctioneer and the expected suprlus to each bidder for both the first price',/, &
 1X,'and second price auction/procurement schemes.',/, &
 1X,' ',/, &
 1X,'The program will ask for the user to provide the following inputs:',/, &
 1X,'-- Whether this is an auctions or procurements environment;',/, &
 1X,'-- The number of types of bidders;',/, &
 1X,'-- The order of the Taylor series expansion to be used.  Based on experience,',/, &
 1X,'   it is recommended that an order of 3 to 5 be used, this order provides very',/, &
 1X,'   cheaply in terms of computation time;',/, &
 1X,'-- The number of subintervals to be used in the backward shooting algorithm.',/, &
 1X,'   Experience shows that 500 to 2000 grid points provide very accurate computation',/, &
 1X,'   of the bid functions;',/, &
 1X,'-- The number of coalitions of each type (k_i in paper);',/, &
 1X,'-- The (common) truncation points of the distribution of private values;',/, &
 1X,'-- The reserve price.  This can set anywhere in the support of the private',/, &
 1X,'   values distributions.  Though this value is not necessary if the user',/, &
 1X,'   wishes to compute the optimal reserve, it still has to be provided;',/, &
 1X,'-- The type of distribution to be used as the distribution of private values.',/, &
 1X,'   There are currently four choices of family of distributions, the two ',/, &
 1X,'   parameter Weibull, the beta, the normal, and the lognormal.',/, &
 1X,'-- The name of the file in which to store the bid functions.  The bid functions',/, &
 1X,'   are stored as arrays of values and corresponding bids for each type. A',/, &
 1X,'   summary of the user defined parameters are also provided in the file.  The',/, &
 1X,'   user should also provide the file extension, for example: bids.txt;',/, &
 1X,'-- The option to use the user defined reserve, or to compute the optimal reserve;',/, &
 1X,'-- The option to provide auctioneer revenue and bidder surplus comparisions;',/, &
 1X,'-- The name of the file in which to store these comparisons.',/, &
 1X,'-- The option to compute best response functions for each bidder.',/, &
 1X,' ',/, &
 1X,'To begin press enter',/, &
 1X,' ',/, &
 1X,' ' )
 READ(*,*)

 WRITE(*,*) ' (1) Enter 1 for auctions, or 2 for procurements: '
 READ(*,*) aucpro

 WRITE(*,*) ' (2) Enter number of h-types: '
 READ(*,*) n
 WRITE(*,*) ' (3) Enter the order of Taylor-series expansion: '
 READ(*,*) bigJ
 WRITE(*,*) ' (4) Enter the number of grid-points: '
 READ(*,*) nt

 ALLOCATE(k(n),u(n),t(0:nt),Fres(n),a(n,0:bigJ),b(n,0:bigJ),c(n,0:bigJ),d(n,0:bigJ), &
          p(n,0:bigJ),q(n,0:bigJ),l(n,0:nt),ll(n,0:nt),lpl(n,0:nt),check(n),bids(n,0:nt),biga(n,n), &
         p0(n,0:nt),bigb(n,1),a1(n,n),a2(n,n),b1(n,n),b2(n,1),c1(bigJ,n),c2(n,n),ta(n,0:nt), &
          tmean(n),tstd(n),lf(n),uf(n),lff(n),uff(n),pf(n),cf(n),pff(n),cff(n),pf1(n),cf1(n),pff1(n),cff1(n))


 WRITE(*,110)
 110 FORMAT (1X,'(5) Enter number of players of each h-type: ')
 READ(*,*) k(:)
 u=1.0d0


 redo=1
 DO !redo

	WRITE(*,121)
	121 FORMAT (1X,'(6) Enter the (common) lower bound of the support of the',/, &
	            1X,'    private-values CDFs: ')
	READ(*,*) lv

	WRITE(*,122)
	122 FORMAT (1X,'(7) Enter the (common) upper bound of the support of the',/, &
	            1X,'    private-values CDFs: ')
	READ(*,*) uv
	
	WRITE(*,*) ' (8) Enter a reserve price: '
	READ(*,*) res

	CALL getcdf
	CALL askcdf
	CALL stats
        IF (aucpro==1) THEN
           WRITE(*,*) "    TYPE     MEAN       ST. DEV     LPDF        UPDF        CRIT"
           DO i=1,n
              WRITE(*,'(1X,I6,2(F12.5),4(D12.2))') i,tmean(i),tstd(i),lf(i),uf(i),lff(i)
           END DO
        ELSE IF (aucpro==2) THEN
           WRITE(*,*) "    TYPE     MEAN       ST. DEV     LPDF        UPDF        CRIT"
           DO i=1,n
              WRITE(*,'(1X,I6,2(F12.5),4(D12.2))') i,tmean(i),tstd(i),lf(i),uf(i),uff(i)
           END DO
        END IF
        WRITE(*,*)
        WRITE(*,123)
        123 FORMAT (1X,'Uniqueness is not guaranteed if any LPDF or UPDF is less than 0.10D-13.',/, &
             1X,'The program is numerically unstable and might even crash for values of CRIT',/, &
             1X,'less than 0.10D-07.  If any of these conditions is violated, you might consider',/, &
             1X,'shortening the support and/or adjusting the parameters of the b-distributions',/, &
             1X,'using the following options: ')
        
        WRITE(*,*)
        WRITE(*,124)
        124 FORMAT (1X,'(13) Enter 1 to continue, 2 to adjust the parameters',/, &
             1X,'     and/or support (return to step 6), or 3 to exit: ')
        READ(*,*) redo
           
        IF (redo==1) EXIT
        IF (redo==3) THEN
           STOP
        END IF

     END DO ! redo

 WRITE(*,130)
 130 FORMAT (1X,'(14) Enter 1 if you wish to compute the optimal reserve.',/, &
             1X,'     Enter 2 if you wish to keep the reserve entered in step 8: ')
 READ(*,*) optres

 WRITE(*,*) '(15) Enter an output file name: '
 READ(*,*) outfile

 ! Construct some necessary stuff for recursion
 bigN = SUM(k)
 sumk = 1.d0/(sum(DBLE(k))-1.0d0)
 DO i=1,n
    b1(:,i)=k(i)
    b1(i,i)=b1(i,i)-1.d0
 END DO

 ! -- Get the timing count rate
 CALL SYSTEM_CLOCK( COUNT_RATE = hertz )

 ! -- Get the start time
 CALL SYSTEM_CLOCK( COUNT = clock0 )

 fires=-10.d0
 sres=-10.d0

 IF (aucpro==1) THEN

 IF (optres==2) THEN  ! option to not compute optimal reserve
    CALL optimt(objs)
    fires=res
    sres=res

 ELSE IF (optres==1) THEN ! option to compute optimal reserve
    CALL optimres(2)
    CALL optimres(1)
    CALL optimt(objs)

 END IF

 ELSE IF (aucpro==2) THEN

 IF (optres==2) THEN  ! option to not compute optimal reserve
    CALL optimt(objs)
    fires=res
    sres=res

 ELSE IF (optres==1) THEN ! option to compute optimal reserve
    CALL poptimres(2)
    CALL poptimres(1)
    CALL optimt(objs)

 END IF

 END IF


 ! -- Get the end time
 CALL SYSTEM_CLOCK( COUNT = clock1 )

 ! -- Output execution time
 time = REAL( clock1 - clock0 ) / REAL( hertz )
 WRITE(*, '( 1x, "Time taken: ",f8.4, "s" )' ) time

 DO i=1,n
    ta(i,:)=t
 END DO

! WRITE(*,*) ' Error in calculation is: ',objs
 WRITE(*,*) ' Writing grid points and bids to file: ', outfile

 IF (objs==1000.0d0) THEN
	WRITE(*,131)
	131 FORMAT (1X,'An error has occured while computing the bid functions.',/, &
             1X,'This is likely to be due to small private values densities',/, &
			 1X,'at the support points.')
	STOP
 END IF

 OPEN (UNIT=8, FILE=outfile,STATUS='REPLACE',ACTION='WRITE')

 DO i=0,nt
    WRITE(8,'(40(2X,F12.8))') ((bids(j,i),ta(j,i)),j=1,n)
 END DO
 CALL printspecs
 CLOSE (UNIT=8)

 WRITE(*,*)
 WRITE(*,140)
 140 FORMAT (1X,'(16) Enter 1 to compute expected revenue and ',/, &
             1X,'     probabilites of winning. Enter 2 otherwise: ')
 READ(*,*) optres

 IF (optres==1) THEN
    WRITE(*,*) '(17) Enter a revenue file name: '
    READ(*,*) outfile

    IF (aucpro==1) THEN
       CALL revenue
    ELSE
       CALL prevenue
    END IF

 END IF

 WRITE(*,150)
 150 FORMAT (1X,'(18) Enter 1 to compute individual best responses.',/, &
             1X,'     Enter 2 otherwise: ')
 READ(*,*) bestr

 IF (bestr==1) THEN
    WRITE(*,*) '(19) Enter a best response file name: '
    READ(*,*) outfile
    IF (aucpro==1) THEN
       CALL bestresponse
    ELSE
       CALL pbestresponse
    END IF
 END IF

 CONTAINS
   SUBROUTINE printspecs
   USE mpreamble
   IMPLICIT NONE

        WRITE(8,*) '  '
        WRITE(8,*) ' Number of coalitions of each type=                  ',k
        WRITE(8,*) ' Number of individuals in each coalition by type=    ',u
        WRITE(8,*) ' Truncated means of distributions of private values =',tmean
        WRITE(8,*) ' Truncated std. of distributions of private values = ',tstd
        WRITE(8,*) ' Lower truncation point =                            ',lv
        WRITE(8,*) ' Upper truncation point =                            ',uv
        WRITE(8,*) ' Reserve Price =                                     ',res
        WRITE(8,*) '  '

   END SUBROUTINE printspecs

END PROGRAM mainprogram

!-----------------------------------------------------------------------------------!

SUBROUTINE optimt(objs)
!***********************************************************************************!
!                          Subroutine OPTIMT                                        !
!-----------------------------------------------------------------------------------!
! Solves for the terminal bid t* corresponding to the equilibrium bid function      !
! Calls the subroutine AMOEBA in which the function fcn1 is provided.               !
! OUTPUT:                                                                           !
! --> tstar (t*)                                                                    !
!***********************************************************************************!
 USE mpreamble
 USE amoebamod
 USE mrecursion
 USE mprecursion
 IMPLICIT NONE
 REAL(8),INTENT(OUT) :: objs
 INTEGER(4)          :: i,j,niter
 REAL(8)             :: obj(ngrid),tt(ngrid),ginc,tstar(1),xint(1),kd(n)
 LOGICAL             :: dogrid
  INTERFACE
     FUNCTION fcn1(x)
     USE mrecursion
     IMPLICIT NONE
     REAL(8),INTENT(IN) :: x(:)
     REAL(8)            :: fcn1
     END FUNCTION fcn1
     FUNCTION pfcn1(x)
     USE mrecursion
     IMPLICIT NONE
     REAL(8),INTENT(IN) :: x(:)
     REAL(8)            :: pfcn1
     END FUNCTION pfcn1
  END INTERFACE 

IF (aucpro==1) THEN

!-- compute the grid of bids (tt) on which to solve for the inverse bids.
 ginc=(uv-res)/dble(ngrid-1)
 tt(1)=res
 DO i=2,ngrid
    tt(i) = tt(i-1)+ginc
 END DO

!-- coarse grid search to locate the neighborhood of the optimum
 obj(1)=1000.d0
 obj(ngrid)=1000.d0
 DO i=2,ngrid-1
    CALL recursion(tt(i),obj(i))
    IF (obj(i)>obj(i-1)) EXIT
 END DO
 tstar=tt(i-1)

!-- call the optimization routine
 xint(1)=ginc
 dogrid=.false.
 CALL amoeba(fcn1,tstar,xint,objs,ftol,niter,dogrid)

ELSE IF (aucpro==2) THEN

!-- compute the grid of bids (tt) on which to solve for the inverse bids.
 ginc=(res-lv)/dble(ngrid-1)
 tt(1)=lv
 DO i=2,ngrid
    tt(i) = tt(i-1)+ginc
 END DO

!-- coarse grid search to locate the neighborhood of the optimum
 obj(1)=1000.d0
 obj(ngrid)=1000.d0
 DO i=ngrid-1,2,-1
    CALL precursion(tt(i),obj(i))
    IF (obj(i)>obj(i+1)) EXIT
 END DO
 tstar=tt(i+1)

!-- call the optimization routine
 xint(1)=ginc
 dogrid=.false.
 CALL amoeba(pfcn1,tstar,xint,objs,ftol,niter,dogrid)

END IF

END SUBROUTINE optimt

!-----------------------------------------------------------------------------------!

FUNCTION fcn1(xx)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Calls the subroutine recursion and returns the objective function (see paper)     !
! for a given trial terminal bid xx                                                 !
! INPUT:                                                                            !
! xx--> trial value of terminal bid                                                 !
! OUTPUT:                                                                           !
! fcn1--> value of the objective function                                           !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
 USE mrecursion
 IMPLICIT NONE
 REAL(8),INTENT(IN)  :: xx(:)
 REAL(8)             :: fcn1
 REAL(8)             :: temp

  CALL recursion(xx(1),temp)
  fcn1=temp
 
 RETURN
END FUNCTION fcn1

FUNCTION pfcn1(xx)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Calls the subroutine recursion and returns the objective function (see paper)     !
! for a given trial terminal bid xx                                                 !
! INPUT:                                                                            !
! xx--> trial value of terminal bid                                                 !
! OUTPUT:                                                                           !
! fcn1--> value of the objective function                                           !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
 USE mprecursion
 IMPLICIT NONE
 REAL(8),INTENT(IN)  :: xx(:)
 REAL(8)             :: pfcn1
 REAL(8)             :: temp

  CALL precursion(xx(1),temp)
  pfcn1=temp
 
 RETURN
END FUNCTION pfcn1

!-----------------------------------------------------------------------------------!

SUBROUTINE optimres(game)
!***********************************************************************************!
!                          Subroutine OPTIMRES                                      !
!-----------------------------------------------------------------------------------!
! Solves for the optimal reserve for the first price and second price auction shemes!
! INPUT:                                                                            !
! game-->  1 for first price auction, 2 for second price auction                    !
! OUTPUT:                                                                           !
! fires--> optimal reserve in first price auction                                   !
! sres-->  optimal reserve in second price auction                                  !
!***********************************************************************************!
 USE mpreamble
 USE cdf_mod
 USE amoebamod1
 IMPLICIT NONE
 INTEGER(4),INTENT(IN) :: game
 REAL(8)               :: xx(1),xint(1),fx,low
 INTEGER(4)            :: tnt,niter
 LOGICAL               :: dogrid
 INTERFACE
   FUNCTION frev(xx)
   USE mpreamble
   IMPLICIT NONE
   REAL(8),INTENT(IN)  :: xx(:)
   REAL(8)             :: frev
   END FUNCTION frev
   FUNCTION srev(xx)
   USE mpreamble
   IMPLICIT NONE
   REAL(8),INTENT(IN)  :: xx(:)
   REAL(8)             :: srev
   END FUNCTION srev
 END INTERFACE


  IF (game==1) THEN    ! First price

  ! reduce the number of grids to speed up algorithm
     tnt=nt
     nt=400

     IF (ALLOCATED(t)) DEALLOCATE(t,bids,l,ll,p0,lpl)
     ALLOCATE(t(0:nt),bids(n,0:nt),l(n,0:nt),ll(n,0:nt),p0(n,0:nt),lpl(n,0:nt))

     xx=0.d0
     xint(1)=(uv-lv)/6.1d0  ! interval to search over
     dogrid=.false.
     CALL amoeba1(frev,xx,xint,fx,tol,niter,dogrid)
     res=lv+(0.85d0*uv-lv)/(1.d0+EXP(-xx(1)))
     fires=res
     nt=tnt

     IF (ALLOCATED(t)) DEALLOCATE(t,bids,l,ll,p0,lpl)
     ALLOCATE(t(0:nt),bids(n,0:nt),l(n,0:nt),ll(n,0:nt),p0(n,0:nt),lpl(n,0:nt))

  ELSE IF (game==2) THEN ! Second price

     xx=0.d0
     xint(1)=(uv-lv)/6.1d0  ! interval to search over
     dogrid=.false.
     CALL amoeba1(srev,xx,xint,fx,tol,niter,dogrid)
     sres=lv+(0.85d0*uv-lv)/(1.d0+EXP(-xx(1)))
  END IF

END SUBROUTINE optimres


SUBROUTINE poptimres(game)
!***********************************************************************************!
!                          Subroutine POPTIMRES                                     !
!-----------------------------------------------------------------------------------!
! Solves for the optimal reserve for the first price and second price auction shemes!
! INPUT:                                                                            !
! game-->  1 for first price auction, 2 for second price auction                    !
! OUTPUT:                                                                           !
! fires--> optimal reserve in first price auction                                   !
! sres-->  optimal reserve in second price auction                                  !
!***********************************************************************************!
 USE mpreamble
 USE cdf_mod
 USE amoebamod1
 IMPLICIT NONE
 INTEGER(4),INTENT(IN) :: game
 REAL(8)               :: xx(1),xint(1),fx,low
 INTEGER(4)            :: tnt,niter
 LOGICAL               :: dogrid
 INTERFACE
   FUNCTION pfrev(xx)
   USE mpreamble
   IMPLICIT NONE
   REAL(8),INTENT(IN)  :: xx(:)
   REAL(8)             :: pfrev
   END FUNCTION pfrev
   FUNCTION psrev(xx)
   USE mpreamble
   IMPLICIT NONE
   REAL(8),INTENT(IN)  :: xx(:)
   REAL(8)             :: psrev
   END FUNCTION psrev
 END INTERFACE


  IF (game==1) THEN    ! First price

  ! reduce the number of grids to speed up algorithm
     tnt=nt
     nt=400

     IF (ALLOCATED(t)) DEALLOCATE(t,bids,l,ll,p0,lpl)
     ALLOCATE(t(0:nt),bids(n,0:nt),l(n,0:nt),ll(n,0:nt),p0(n,0:nt),lpl(n,0:nt))

     xx=0.0d0
     xint(1)=(uv-lv)/6.1d0  ! interval to search over
     dogrid=.false.
     CALL amoeba1(pfrev,xx,xint,fx,tol,niter,dogrid)
     res=1.15d0*lv+(uv-1.15d0*lv)/(1.d0+EXP(-xx(1)))
     fires=res
     nt=tnt

     IF (ALLOCATED(t)) DEALLOCATE(t,bids,l,ll,p0,lpl)
     ALLOCATE(t(0:nt),bids(n,0:nt),l(n,0:nt),ll(n,0:nt),p0(n,0:nt),lpl(n,0:nt))

  ELSE IF (game==2) THEN ! Second price

     xx=0.0d0
     xint(1)=(uv-lv)/6.1d0  ! interval to search over
     dogrid=.false.
     CALL amoeba1(psrev,xx,xint,fx,tol,niter,dogrid)
     sres=1.15d0*lv+(uv-1.15d0*lv)/(1.d0+EXP(-xx(1)))
  END IF

END SUBROUTINE poptimres

!-----------------------------------------------------------------------------------!

SUBROUTINE revenue
!***********************************************************************************!
!                          Subroutine REVENUE                                       !
!-----------------------------------------------------------------------------------!
! Computes the revenue to the auctioneer and the surplus to each bidder in the      !
! first price and second price auction schemes                                      !
!***********************************************************************************!
 USE mpreamble
 USE quadrature
 USE cdf_mod
 IMPLICIT NONE
 REAL(8)             :: frev(1),ssrev,mult(0:nt),fsurp(n),ssurp(n),cdfres(n)
 REAL(8)             :: qa(nq),qw(nq),set,set1(nq,n),set2(nq,n),set3(nq,n), &
                        set4(nq),set5(nq,n),low1(nq,n),setr(n),settr(n),low(nq,n), &
                        fprob(0:n),sprob(0:n)
 INTEGER(4)          :: i,j,h,m

 !-- FIRST PRICE AUCTION
 !----------------------

 ! compute the weight for integration by the extended Simpson rule
  inc=t(nt)-t(nt-1)
  mult(0)=0.0d0
  mult(1)=inc*3.0d0/2.0d0
  DO i=2,nt-1
     mult(i)=inc
  END DO
  mult(nt)=inc/2.0d0

  res=fires

  OPEN (UNIT=8, FILE=outfile,STATUS='REPLACE',ACTION='WRITE')

 ! revenue to auctioneer (see paper for formula)

  DO i=1,n
     ll(i,:)=l(i,:)**k(i)
  END DO
  cdfres=f_cdf(fires)**k

  frev=t(nt)-fires*PRODUCT(cdfres)-SUM(mult*PRODUCT(ll,DIM=1))

 ! per capita surplus for each coalition and probability of winning
  DO i=1,n
     fsurp(i) = SUM(mult*p0(i,:)*lpl(i,:)*PRODUCT(ll,DIM=1))/DBLE(u(i))
     fprob(i) = SUM(mult*lpl(i,:)*PRODUCT(ll,DIM=1))/DBLE(u(i))
  END DO

 ! probability of no wins
 fprob(0) = PRODUCT(f_cdf(fires)**k)


 !-- SECOND PRICE AUCTION
 !-----------------------

  res=sres

 ! get Gauss-Legendre quadrature abscissa and weights for integration
  CALL gauleg(sres,uv,qa,qw)

  set2=1.d0
  DO i=1,nq
     low(i,:)=f_cdf(qa(i))
     low1(i,:)=f_pdf(qa(i))
     DO j=1,n
        set1(i,j)=(1.d0-low(i,j))
        DO h=1,n
           IF (h==j) THEN
              set2(i,j)=set2(i,j)*low(i,h)**(k(h)-1)
           ELSE
              set2(i,j)=set2(i,j)*low(i,h)**k(h)
           END IF
        END DO
        set2(i,j)=set2(i,j)*qw(i)
        set3(i,j)=DBLE(k(j))*set1(i,j)*set2(i,j)
        set5(i,j)=low1(i,j)*set2(i,j)
     END DO
     set4(i)=PRODUCT(f_cdf(qa(i))**k)*qw(i)
     low(i,:)=low(i,:)*qw(i)
  END DO

 ! per capita surplus for each coalition
  setr=1.d0
  DO i=1,n
     settr=f_cdf(sres)
     DO j=1,n
        IF (j==i) THEN
           setr(i)=setr(i)*settr(j)**(k(j)-1)
           ELSE
           setr(i)=setr(i)*settr(j)**k(j)
        END IF
     END DO
  END DO

  ssurp = SUM(set3,DIM=1)/DBLE(u*k)

 ! probability of winning
  sprob(1:n) = SUM(set5,DIM=1)/DBLE(u)

 ! probability of no wins
  sprob(0) = PRODUCT(f_cdf(sres)**k)

 ! revenue to auctioneer (see paper for formula)
  set = SUM(set3)
  ssrev = uv-sres*PRODUCT(f_cdf(sres)**k)-SUM(set4)-set


  WRITE(8,*) ' FIRST PRICE AUCTION'
  WRITE(8,*) ' -------------------'
  WRITE(8,'(1X, "EXPECTED REVENUE:        ",2X,F10.6)') frev
  WRITE(8,'(1X, "EXPECTED SURPLUS:        ",10(2X,F10.6))') fsurp
  WRITE(8,'(1X, "PROBABILITY OF WINNING:  ",10(2X,F10.6))') fprob(1:n)
  WRITE(8,'(1X, "PROBABILITY OF NO WINS:  ",2X,F10.6)') fprob(0)
  WRITE(8,'(1X, "OPTIMAL RESERVE:         ",2X,F10.6)') fires
  WRITE(8,*) ' '
  WRITE(8,*) ' SECOND PRICE AUCTION'
  WRITE(8,*) ' --------------------'
  WRITE(8,'(1X, "EXPECTED REVENUE:        ",2X,F10.6)') ssrev
  WRITE(8,'(1X, "EXPECTED SURPLUS:        ",10(2X,F10.6))') ssurp
  WRITE(8,'(1X, "PROBABILITY OF WINNING:  ",10(2X,F10.6))') sprob(1:n)
  WRITE(8,'(1X, "PROBABILITY OF NO WINS:  ",2X,F10.6)') sprob(0)
  WRITE(8,'(1X, "OPTIMAL RESERVE:         ",2X,F10.6)') sres

  CLOSE(UNIT=8)

END SUBROUTINE revenue


SUBROUTINE prevenue
!***********************************************************************************!
!                          Subroutine PREVENUE                                      !
!-----------------------------------------------------------------------------------!
! Computes the revenue to the auctioneer and the surplus to each bidder in the      !
! first price and second price auction schemes                                      !
!***********************************************************************************!
 USE mpreamble
 USE quadrature
 USE cdf_mod
 IMPLICIT NONE
 REAL(8)             :: frev(1),ssrev,mult(0:nt),fsurp(n),ssurp(n),cdfres(n)
 REAL(8)             :: qa(nq),qw(nq),set,set1(nq,n),set2(nq,n),set3(nq,n), &
                        set4(nq),set5(nq,n),low1(nq,n),setr(n),settr(n),low(nq,n), &
                        fprob(0:n),sprob(0:n)
 INTEGER(4)          :: i,j,h,m

 !-- FIRST PRICE AUCTION
 !----------------------

 ! compute the weight for integration by the extended Simpson rule
  inc=t(1)-t(0)
  mult(0)=0.0d0
  mult(1)=inc*3.0d0/2.0d0
  DO i=2,nt-1
     mult(i)=inc
  END DO
  mult(nt)=inc/2.0d0

  res=fires

  OPEN (UNIT=8, FILE=outfile,STATUS='REPLACE',ACTION='WRITE')

 ! revenue to auctioneer (see paper for formula)

  DO i=1,n
     ll(i,:)=l(i,:)**k(i)
  END DO
  cdfres=(1.0d0-f_cdf(fires))**k

  frev=t(0)-fires*PRODUCT(cdfres)+SUM(mult*PRODUCT(ll,DIM=1))

 ! per capita surplus for each coalition and probability of winning
  DO i=1,n
     fsurp(i) = -SUM(mult*p0(i,:)*lpl(i,:)*PRODUCT(ll,DIM=1))/DBLE(u(i))
     fprob(i) = SUM(mult*lpl(i,:)*PRODUCT(ll,DIM=1))/DBLE(u(i))
  END DO

 ! probability of no wins
 fprob(0) = PRODUCT((1.0d0-f_cdf(fires))**k)


 !-- SECOND PRICE AUCTION
 !-----------------------

  res=sres

 ! get Gauss-Legendre quadrature abscissa and weights for integration
  CALL gauleg(lv,sres,qa,qw)

  set2=1.d0
  DO i=1,nq
     low(i,:)=1.0d0-f_cdf(qa(i))
     low1(i,:)=-f_pdf(qa(i))
     DO j=1,n
        set1(i,j)=(1.d0-low(i,j))
        DO h=1,n
           IF (h==j) THEN
              set2(i,j)=set2(i,j)*low(i,h)**(k(h)-1)
           ELSE
              set2(i,j)=set2(i,j)*low(i,h)**k(h)
           END IF
        END DO
        set2(i,j)=set2(i,j)*qw(i)
        set3(i,j)=DBLE(k(j))*set1(i,j)*set2(i,j)
        set5(i,j)=low1(i,j)*set2(i,j)
     END DO
     set4(i)=PRODUCT((1.0d0-f_cdf(qa(i)))**k)*qw(i)
     low(i,:)=low(i,:)*qw(i)
  END DO

 ! per capita surplus for each coalition
  setr=1.d0
  DO i=1,n
     settr=1.0d0-f_cdf(sres)
     DO j=1,n
        IF (j==i) THEN
           setr(i)=setr(i)*settr(j)**(k(j)-1)
           ELSE
           setr(i)=setr(i)*settr(j)**k(j)
        END IF
     END DO
  END DO

  ssurp = SUM(set3,DIM=1)/DBLE(u*k)

 ! probability of winning
  sprob(1:n) = -SUM(set5,DIM=1)/DBLE(u)

 ! probability of no wins
  sprob(0) = PRODUCT((1.0d0-f_cdf(sres))**k)

 ! revenue to auctioneer (see paper for formula)
  set = SUM(set3)
  ssrev = lv-sres*PRODUCT((1.0d0-f_cdf(sres))**k)+SUM(set4)+set


  WRITE(8,*) ' FIRST PRICE AUCTION'
  WRITE(8,*) ' -------------------'
  WRITE(8,'(1X, "EXPECTED REVENUE:        ",2X,F10.6)') frev
  WRITE(8,'(1X, "EXPECTED SURPLUS:        ",10(2X,F10.6))') fsurp
  WRITE(8,'(1X, "PROBABILITY OF WINNING:  ",10(2X,F10.6))') fprob(1:n)
  WRITE(8,'(1X, "PROBABILITY OF NO WINS:  ",2X,F10.6)') fprob(0)
  WRITE(8,'(1X, "OPTIMAL RESERVE:         ",2X,F10.6)') fires
  WRITE(8,*) ' '
  WRITE(8,*) ' SECOND PRICE AUCTION'
  WRITE(8,*) ' --------------------'
  WRITE(8,'(1X, "EXPECTED REVENUE:        ",2X,F10.6)') ssrev
  WRITE(8,'(1X, "EXPECTED SURPLUS:        ",10(2X,F10.6))') ssurp
  WRITE(8,'(1X, "PROBABILITY OF WINNING:  ",10(2X,F10.6))') sprob(1:n)
  WRITE(8,'(1X, "PROBABILITY OF NO WINS:  ",2X,F10.6)') sprob(0)
  WRITE(8,'(1X, "OPTIMAL RESERVE:         ",2X,F10.6)') sres

  CLOSE(UNIT=8)

END SUBROUTINE prevenue
!-----------------------------------------------------------------------------------!

FUNCTION frev(xx)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! returns the auctioneer revenue in first price auction                             !
! for a given trial terminal bid xx                                                 !
! INPUT:                                                                            !
! xx--> trial value of revenue                                                      !
! OUTPUT:                                                                           !
! fcn1--> value of the auctioneer revenue                                           !
! details are found in subroutine REVENUE                                           !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
 USE mpreamble
 USE cdf_mod
 IMPLICIT NONE
 REAL(8),INTENT(IN)  :: xx(:)
 REAL(8)             :: frev
 REAL(8)             :: mult(0:nt),objj,cdfres(n)
 INTEGER(4)          :: i

  res=lv+(0.85d0*uv-lv)/(1.d0+EXP(-xx(1)))
  fires=res
  CALL optimt(objj)
  inc=t(nt)-t(nt-1)
  mult(0)=0.0d0
  mult(1)=inc*3.0d0/2.0d0
  DO i=2,nt-1
     mult(i)=inc
  END DO
  mult(nt)=inc/2.0d0

  DO i=1,n
     ll(i,:)=l(i,:)**k(i)
  END DO
  cdfres=f_cdf(fires)**k

 frev=-(t(nt)-fires*PRODUCT(cdfres)-SUM(mult*PRODUCT(ll,DIM=1)))

END FUNCTION frev

FUNCTION pfrev(xx)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! returns the auctioneer revenue in first price auction                             !
! for a given trial terminal bid xx                                                 !
! INPUT:                                                                            !
! xx--> trial value of revenue                                                      !
! OUTPUT:                                                                           !
! fcn1--> value of the auctioneer revenue                                           !
! details are found in subroutine REVENUE                                           !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
 USE mpreamble
 USE cdf_mod
 IMPLICIT NONE
 REAL(8),INTENT(IN)  :: xx(:)
 REAL(8)             :: pfrev
 REAL(8)             :: mult(0:nt),objj,cdfres(n)
 INTEGER(4)          :: i

  res=1.15d0*lv+(uv-1.15d0*lv)/(1.d0+EXP(-xx(1)))
  fires=res
  CALL optimt(objj)
  inc=t(nt)-t(nt-1)
  mult(0)=0.0d0
  mult(1)=inc*3.0d0/2.0d0
  DO i=2,nt-1
     mult(i)=inc
  END DO
  mult(nt)=inc/2.0d0

  DO i=1,n
     ll(i,:)=l(i,:)**k(i)
  END DO
  cdfres=(1.0d0-f_cdf(fires))**k

  pfrev=-(t(0)-fires*PRODUCT(cdfres)+SUM(mult*PRODUCT(ll,DIM=1)))

END FUNCTION pfrev


!-----------------------------------------------------------------------------------!

FUNCTION srev(xx)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! returns the auctioneer revenue in second price auction                            !
! for a given trial terminal bid xx                                                 !
! INPUT:                                                                            !
! xx--> trial value of revenue                                                      !
! OUTPUT:                                                                           !
! fcn1--> value of the auctioneer revenue                                           !
! details are found in subroutine REVENUE                                           !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
 USE mpreamble
 USE quadrature
 USE cdf_mod
 IMPLICIT NONE
 REAL(8),INTENT(IN)  :: xx(:)
 REAL(8)             :: srev
 REAL(8)             :: mult(0:nt),objj,qa(100),qw(100), &
                        set,set1(100,n),set2(100,n),set3(100,n),set4(100),low(100,n)
 INTEGER(4)          :: i,j,h

  sres=lv+(0.85d0*uv-lv)/(1.d0+EXP(-xx(1)))
  CALL gauleg(sres,uv,qa,qw)


  set2=1.d0
  DO i=1,100
     low(i,:)=f_cdf(qa(i))
     DO j=1,n
        set1(i,j)=(1.d0-low(i,j))
        DO h=1,n
           IF (h==j) THEN
              set2(i,j)=set2(i,j)*low(i,h)**(k(h)-1)
           ELSE
              set2(i,j)=set2(i,j)*low(i,h)**k(h)
           END IF
        END DO
        set3(i,j)=DBLE(k(j))*set1(i,j)*set2(i,j)*qw(i)
        set4(i)=PRODUCT(f_cdf(qa(i))**k)*qw(i)
     END DO
  END DO

  set=SUM(set3)
  srev=-(uv-sres*PRODUCT(f_cdf(sres)**k)-SUM(set4)-set)

END FUNCTION srev


FUNCTION psrev(xx)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! returns the auctioneer revenue in second price auction                            !
! for a given trial terminal bid xx                                                 !
! INPUT:                                                                            !
! xx--> trial value of revenue                                                      !
! OUTPUT:                                                                           !
! fcn1--> value of the auctioneer revenue                                           !
! details are found in subroutine REVENUE                                           !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
 USE mpreamble
 USE quadrature
 USE cdf_mod
 IMPLICIT NONE
 REAL(8),INTENT(IN)  :: xx(:)
 REAL(8)             :: psrev
 REAL(8)             :: mult(0:nt),objj,qa(100),qw(100), &
                        set,set1(100,n),set2(100,n),set3(100,n),set4(100),low(100,n)
 INTEGER(4)          :: i,j,h

  sres=1.15d0*lv+(uv-1.15d0*lv)/(1.d0+EXP(-xx(1)))
  CALL gauleg(lv,sres,qa,qw)


  set2=1.d0
  DO i=1,100
     low(i,:)=1.0d0-f_cdf(qa(i))
     DO j=1,n
        set1(i,j)=(1.0d0-low(i,j))
        DO h=1,n
           IF (h==j) THEN
              set2(i,j)=set2(i,j)*low(i,h)**(k(h)-1)
           ELSE
              set2(i,j)=set2(i,j)*low(i,h)**k(h)
           END IF
        END DO
        set3(i,j)=DBLE(k(j))*set1(i,j)*set2(i,j)*qw(i)
        set4(i)=PRODUCT((1.0d0-f_cdf(qa(i)))**k)*qw(i)
     END DO
  END DO

  set=SUM(set3)
  psrev=-(lv-sres*PRODUCT((1.0d0-f_cdf(sres))**k)+SUM(set4)+set)

END FUNCTION psrev


SUBROUTINE bestresponse
!**********************************************************************************!
!                      Subroutine BESTRESPONSE                                     !
!----------------------------------------------------------------------------------!
! For each bidder i, this subroutine computes the best response to the optimal     !
! strategies of all other bidders.  It does so by solving for each bidder the      !
! first order condition (equation 3 in the paper).  These best responses can be    !
! used to check the numerical accuracy of the algorithm since if the algorithm     !
! provides accurate bid functions, the best responses should be the same as these  !
! bid functions.                                                                   !
!**********************************************************************************!
USE mpreamble
IMPLICIT NONE
INTEGER(4)            :: i,j,h,ns,l0,ml(1)
REAL(8)               :: xx,f1,fu,fl,incc

 ALLOCATE(mat1(0:nt),mat0(0:nt),vgrid(0:nt),brr(2*n,0:nt),kstar(n),slpl(0:nt))


! Create an equally spaced grid on which to compute best responses
 vgrid(0)=fires
 vgrid(nt)=uv
 incc=(vgrid(nt)-vgrid(0))/DBLE(nt)
 DO ns=1,nt-1
    vgrid(ns)=vgrid(ns-1)+incc
 END DO

 h=1
 DO i=1,n

    kstar=DBLE(k)
    kstar(i)=kstar(i)-1.0d0  ! kstar as in paper

    slpl=0.0d0
    DO j=1,n
      slpl= slpl+kstar(j)*lpl(j,:)
    END DO

! Find best response
    DO ns=1,nt-1
       xx=vgrid(ns)
       mat0 = (1.0d0-(xx-t)*slpl)
       mat1 = mat0**2
       ml= MINLOC(mat1)
       brr(h,ns)=xx
       brr(h+1,ns)=t(ml(1))
    END DO

    h=h+2
 END DO

 OPEN (UNIT=12,FILE=outfile,ACTION='write')
 DO i=1,nt-1
    WRITE(12,'(40(2x,F12.8))') brr(:,i)
 END DO
 CLOSE (UNIT=12)

END SUBROUTINE bestresponse


SUBROUTINE pbestresponse
!**********************************************************************************!
!                      Subroutine PBESTRESPONSE                                    !
!----------------------------------------------------------------------------------!
! Same as BESTRESPONSE, but now for procurements                                   !
!**********************************************************************************!
USE mpreamble
IMPLICIT NONE
INTEGER(4)            :: i,j,h,ns,l0,ml(1)
REAL(8)               :: xx,f1,fu,fl,incc

 ALLOCATE(mat1(0:nt),mat0(0:nt),vgrid(0:nt),brr(2*n,0:nt),kstar(n),slpl(0:nt))


! Create an equally spaced grid on which to compute best responses
 vgrid(0)=lv
 vgrid(nt)=fires
 incc=(vgrid(nt)-vgrid(0))/DBLE(nt)
 DO ns=1,nt-1
    vgrid(ns)=vgrid(ns-1)+incc
 END DO

 h=1
 DO i=1,n

    kstar=DBLE(k)
    kstar(i)=kstar(i)-1.0d0

    slpl=0.0d0
    DO j=1,n
      slpl= slpl+kstar(j)*lpl(j,:)
    END DO

! Find best response
    DO ns=1,nt-1
       xx=vgrid(ns)
       mat0 = (1.0d0-(t-xx)*slpl)
       mat1 = mat0**2
       ml= MINLOC(mat1)
       brr(h,ns)=xx
       brr(h+1,ns)=t(ml(1))
    END DO

    h=h+2
 END DO

 OPEN (UNIT=12,FILE=outfile,ACTION='write')
 DO i=1,nt-1
    WRITE(12,'(40(2x,F12.8))') brr(:,i)
 END DO
 CLOSE (UNIT=12)

END SUBROUTINE pbestresponse



