!     Last change:  WRG  18 Jul 2006    9:48 pm
MODULE mpreamble
!================================================================================!
! **** LAST MODIFIED: 10 Feb 2005 ****                                           !
! **** WAYNE-ROY GAYLE, JEAN-FRANCOIS RICHARD, UNIVERSITY OF PITTSBURGH ****     !
! This module stores key parameters, that are used in multible subroutines and   !
! functions                                                                      !
! KEY DEFINITIONS:                                                               !
! n     : number of types of players                                             !
! k     : (n*1)-dimensional vector of number of coalitions of each type          !
! u     : (n*1)-dimensional vector of the number of bidders in each coalition    !
! bigN  : number of actual bidders (bigN=sum of k_i,i=1,...,n)                   !
! bidJ  : order of taylor series expansions.                                     !
! uv    : upper support point of distribution function                           !
! lv    : lower support point of distribution function                           !
! res   : generic reserve price                                                  !
! fires : stores optimal reserve in first price auction                          !
! sres  : stores optimal reserve in second price auction                         !
! a     : (n*bigJ) matrix of taylor series coefficients of l                     !
! b     : (n*bigJ) matrix of taytor series coefficients of l'/l                  !
! dtype : stores the choice of distribution                                      !
! tol   : tolerance used in optimzation to compute optimal reserve               !
! ftol  ! tolerance used in the computation of t* (see paper)                    !
!================================================================================!
IMPLICIT NONE

 INTEGER(4),PARAMETER       :: ngrid=60,npt=1001,ko=6,nko=npt+ko,nq=200
 REAL(8),PARAMETER          :: tol=1d-3,ftol=1d-6
 INTEGER(4),SAVE            :: n,bigN,bigJ,nt,dtype,nquad,npr,npc,npr1,npc1,ncdf,aucpro
 REAL(8),SAVE               :: uv,lv,res,sumk,fires,sres,aa,bb,inc
 INTEGER(4),ALLOCATABLE,DIMENSION(:)  :: k,check,icdf
 REAL(8),ALLOCATABLE,DIMENSION(:,:)   :: qcdf
 REAL(8),ALLOCATABLE,DIMENSION(:)     :: u,t,ccu,ccl,Fres,tmean,tstd,mat1,kstar, &
                                         mat0,vgrid,slpl,lf,uf,lff,uff,pf,cf,pff,cff,pf1,cf1,pff1,cff1
 REAL(8),ALLOCATABLE,DIMENSION(:,:)   :: a,b,c,d,p,p0,q,l,lpl,bids,biga,bigb, &
                                         a1,a2,b1,b2,c1,c2,ll,bcf,br,bcf1,br1, &
                                         musd,brr
 REAL(8),ALLOCATABLE,DIMENSION(:,:,:) :: pcf,pcf1
 CHARACTER(LEN=40)                    :: outfile

END MODULE mpreamble
